//*******************************************
// Simplest vertex and fragment shaders
// ******************************************
// Sets the position and color of a vertex.
//   This implementations just copies the position with no transformations.
//   It copies the color to "theColor" so that the fragment shader can access it.

#beginglsl vertexshader vertexShader_Chap1Figs
#version 330 core
layout (location = 0) in vec3 vertPos;	   // Position in attribute location 0
layout (location = 1) in vec3 vertColor;   // Color in attribute location 1
out vec3 theColor;					       // Output a color to the fragment shader
void main()
{
   gl_Position = vec4(vertPos.x-1.0, 0.666*vertPos.y-1.0, vertPos.z, 1.0);
   theColor = vertColor;
}
#endglsl

// Set a general color using a fragment shader. (A "fragment" is a "pixel".)
//    The color value is passed in, obtained from the color(s) on the vertice(s).
//    Color values range from 0.0 to 1.0.
//    First three values are Red/Green/Blue (RGB).
//    Fourth color value (alpha) is 1.0, meaning there is no transparency.

#beginglsl FragmentShader fragmentShader_Chap1Figs
#version 330 core
in vec3 theColor;		// Color value came from the vertex shader (smoothed) 
out vec4 FragColor;	    // Color that will be used for the fragment
void main()
{
   FragColor = vec4(theColor, 1.0f);   // Add alpha value of 1.0.
}
#endglsl

// ********************************************
// Flat versions
// ********************************************
// vertexShader_FlatChap1Figs
//    Same as vertexShader_Chap1Figs but with flat shading
#beginglsl vertexshader vertexShader_FlatChap1Figs
#version 330 core
layout (location = 0) in vec3 vertPos;	   // Position in attribute location 0
layout (location = 1) in vec3 vertColor;   // Color in attribute location 1
flat out vec3 theColor;					       // Output a color to the fragment shader
void main()
{
   gl_Position = vec4(vertPos.x-1.0, 0.666*vertPos.y-1.0, vertPos.z, 1.0);
   theColor = vertColor;
}
#endglsl

// fragmentShader_FlatChap1Figs:
//    Same as fragmentShader_ColorOnly, but with flat shading
#beginglsl FragmentShader fragmentShader_FlatChap1Figs
#version 330 core
flat in vec3 theColor;		// Color value came from the vertex shader (smoothed) 
out vec4 FragColor;	    // Color that will be used for the fragment
void main()
{
   FragColor = vec4(theColor, 1.0f);   // Add alpha value of 1.0.
}
#endglsl

//*******************************************
// Fragment shader that raises all colors to maximum brightness (Value)
//    Use this with vertexShader_Chap1Figs
// ******************************************
#beginglsl FragmentShader fragmentShader_BrightColor
#version 330 core
in vec3 theColor;		// Color value came from the vertex shader (smoothed) 
out vec4 FragColor;	    // Color that will be used for the fragment
void main()
{
    float mx = max(theColor.r,max(theColor.g,theColor.b)); 
    if ( mx!=0 ) {
        FragColor = vec4(theColor/mx, 1.0f);   // Scale max component to 1.0.
    }
	else {
	    FragColor = vec4(1.0, 1.0, 1.0, 1.0);	// Replace black with white
	}
}
#endglsl

